{% load cache %}
<!DOCTYPE html>
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <title>Bad form</title>
  </head>
  <body>
    <h1>Bad form test</h1>
    <form>
      <input type="file" name="file" />
    </form>
  </body>
</html>
