{% load i18n %}
<table>
  <thead>
    <tr>
      <th>{% translate "Signal" %}</th>
      <th>{% translate "Receivers" %}</th>
    </tr>
  </thead>
  <tbody>
    {% for name, receivers in signals %}
      <tr>
        <td>{{ name|escape }}</td>
        <td>{{ receivers|join:", " }}</td>
      </tr>
    {% endfor %}
  </tbody>
</table>
