<!DOCTYPE html>
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <title>Cache</title>
  </head>
  <body>
    <h1>Cache Test</h1>
    <p>Check the cache panel to see the cache hits and misses.</p>
  </body>
</html>
