{% load i18n %}

<div class="djdt-community-panel">
    <h2>
        {% translate "Community & Contribution" %}
    </h2>
    <p>
        {% translate "Want to contribute to Django Debug Toolbar? Get involved in our community!" %}
    </p>

    <ul>
        <li>
            <a href="https://github.com/django-commons/django-debug-toolbar/discussions" target="_blank">
                {% translate "Join Discussions" %}
            </a>
        </li>
        <li>
            <a href="https://github.com/django-commons/django-debug-toolbar/issues" target="_blank">
                {% translate "View Issues" %}
            </a>
        </li>
        <li>
            <a href="https://django-debug-toolbar.readthedocs.io/en/latest/contributing.html" target="_blank">
                {% translate "Contribution Guide" %}
            </a>
        </li>
    </ul>
    <hr>
    <br>
    <h2>
        {% translate "Django Debug Toolbar Documentation" %}
    </h2>
    <p class="djdt-community-description">
        {% translate "Explore the official documentation to learn more about Django Debug Toolbar." %}
    </p>
    <ul>
        <li>
            <a href="https://django-debug-toolbar.readthedocs.io/en/latest/" target="_blank">
                {% translate "Read Documentation" %}
            </a>
        </li>
        <li>
            <a href="https://django-debug-toolbar.readthedocs.io/en/latest/resources.html" target="_blank">
                {% translate "How to Use Django Debug Toolbar" %}
            </a>
        </li>
    </ul>
</div>
