{% load i18n %}
<table>
  <thead>
    <tr>
      <th>{% translate "Setting" %}</th>
      <th>{% translate "Value" %}</th>
    </tr>
  </thead>
  <tbody>
    {% for name, value in settings.items %}
      <tr>
        <td>{{ name }}</td>
        <td><code>{{ value }}</code></td>
      </tr>
    {% endfor %}
  </tbody>
</table>
